// setup namespace objects
if(!com) {
	var com = {};
}
if(!com.ibm) {
	com.ibm = {};
}
if(!com.ibm.rpe) {
	com.ibm.rpe = {};
}

if(!com.ibm.rpe.DojoHelper) {
	com.ibm.rpe.DojoHelper = function() {}
}


// the helper object is a singleton, so just define the "static methods" directly as member functions (no prototypes needed)

com.ibm.rpe.DojoHelper.STATE_SELECTED_INDEX = "[idx]";
com.ibm.rpe.DojoHelper.STATE_VISIBLE_MOBILE_VIEW = "[vmv]";

// Finds the next-closest widget above the given node in the DOM that maps to an equivalent source node.  This function skips over
// any widgets that were created implicitly (ex. extra container widgets for layout dijits, etc.)
com.ibm.rpe.DojoHelper.findParentWidget = function(node) {
	if(node && node.parentNode && (typeof(dijit) != 'undefined')) {
		var parentWidget = dijit.getEnclosingWidget(node.parentNode);
		while(parentWidget && !com.ibm.rpe.Editor.getNodeId(parentWidget.domNode)) {
			// keep going up the DOM until we find a widget that has an equivalent in source.
			// we check this by looking for our magic editor ID, since non-source nodes won't have this DOM node attribute
			parentWidget = dijit.getEnclosingWidget(parentWidget.domNode.parentNode);
		}
		return parentWidget;
	}
	return null;
}



// Programatically creates a new Dojo widget in the DOM.  This function is called by a script that's embedded in the visualization DOM.
com.ibm.rpe.DojoHelper.createDojoWidget = function(nodeId, nextSiblingId, dojoType, paramsString) {
	var createFunc = function () {
		// first check for dojo/dijit
		if(typeof(dijit) != 'undefined') {
			// we need to put the creation code in an addOnLoad function because the prereq dojo.require() calls may still be finishing
			// first destroy an old widget with this ID, if present (though it shouldn't be if the removal events all processed correctly)
			var oldWidget = dijit.byId(nodeId);
			if(oldWidget) {
				com.ibm.rpe.DojoHelper.destroyWidgetForReplacement(oldWidget);
			}

			// find the new div where we're creating this node
			var newDiv = document.getElementById(nodeId);
			if(newDiv) {
				var parentNode = newDiv.parentNode;
				var parentWidget = com.ibm.rpe.DojoHelper.findParentWidget(newDiv);

				// create the object of widget parameters from the passed string
				var params = dojo.fromJson(paramsString);
				com.ibm.rpe.DojoHelper.addWidgetSpecificParams(dojoType, newDiv, params);
				// now tweak the params object to fix the parameter types to match what the widget constructor is expecting
				com.ibm.rpe.DojoHelper.fixupWidgetParams(params, dojoType);
				// create the widget
				var widgetCreatorFunc = new Function('paramsObj', 'nodeId', 'return new '+dojoType+'(paramsObj, nodeId);');
				try {
					var widget = widgetCreatorFunc(params, nodeId);
				} catch (e) {
					//In some cases (some Dojox widgets like all the dojox.dgauges), the nodeId is not recognized and causes an error, 
					//for these cases, we look for the element with the specified id in the document before call widget creator function.
					var widget = widgetCreatorFunc(params, document.getElementById(nodeId));
				}
				if (dojoType == "dojox.mobile.TabBar" && dojo.hasClass(parentNode, 'mblScrollableView')) {
					// There isn't a programmatic way to fix the tabbar to the bottom, and refreshing the page is jarring
					(function(fixed) {
						if (fixed == "bottom") {
							dojo.addClass(newDiv, "mblFixedBottomBar");
							if (dojo.style(newDiv, "bottom") == 'auto') {
								dojo.style(newDiv, "bottom", "0px");
							}
						}
						else if (fixed == "top") {
							dojo.addClass(newDiv, "mblFixedHeaderBar");
							if (dojo.style(newDiv, "top") == 'auto') {
								dojo.style(newDiv, "top", "0px");
							}
						}
					})(newDiv.getAttribute("fixed"));
				}
				// also set the node id directly on the created dom element so that we can find it later
				var domNode = widget.domNode;
				if(domNode) {
					com.ibm.rpe.Editor.setNodeId(domNode, nodeId);
				}

				// for a widget going inside certain layout containers, we need to programmatically add the children to the container widget
				if(parentWidget && parentWidget.addChild && dojoType != 'dojox.mobile.TabBar') {
					// figure out the proper index to do the insertion
					var children = parentWidget.getChildren();

					// first see if we've already added the widget in a good location
					var index = com.ibm.rpe.DojoHelper.indexOf(children, widget);
					if(index < 0) {
						// we didn't find our widget as a child of our intended parent (probably because it got contructed in the DOM under an extra container node)
						// so go calculate the index relative to the next sibling node
						var nextSiblingNode = com.ibm.rpe.Editor.findNodeById(nextSiblingId);
						if(nextSiblingNode) {
							var nextSiblingWidget = dijit.getEnclosingWidget(nextSiblingNode);
							if(nextSiblingWidget) {
								// because this widget is being added before the next sibling, the sibling's index will become our new index
								index = com.ibm.rpe.DojoHelper.indexOf(children, nextSiblingWidget);
							}
						}
					}
					// add the widget into the parent container. If the index is negative, just add the widget at the beginning
					if(index < 0) {
						widget.placeAt(parentWidget);
					} else {
						widget.placeAt(parentWidget, index);
					}
				}
			}
		}
	};

	if(com.ibm.rpe.DojoFrameworkOverrides.usingAsyncLoader) {
		require(["dojo/ready"], 
				function(ready) {
					ready(createFunc);
				
		});	
	} else {
		dojo.addOnLoad(createFunc);
	}

}

// Handles special cases where certain widgets may need to add to the constructor's
// parameters.
com.ibm.rpe.DojoHelper.addWidgetSpecificParams = function(dojoType, node, params) {
	if (dojoType === "dojox.grid.DataGrid") {
		// With a declarative DataGrid, the structure is defined by the <thead> element
		// Passing this table to the widget constructor, however, does not seem to infer
		// the structure from that element. This creates the 'structure' parameter by
		// extracting the information from <thead>
		var structure = [];
		var children = node.childNodes, tr, th;
		var i, j, k;
		var cell;
		var fieldWidth;

		/* Transfer an attribute property to the data object. If missing, use a default */
		var transferProperty = function(data, item, attr, def) {
			var prop = item.getAttribute(attr);
			if (prop) {
				data[attr] = prop;
			}
			else if (def) {
				data[attr] = def;
			}
		};

		for (i = 0; i < children.length; i++) {
			if (children[i].nodeName.toLowerCase() == 'thead') {
				tr = children[i].childNodes;
				for (j = 0; j < tr.length; j++) {
					if (tr[j].nodeName.toLowerCase() == 'tr') {
						structure.push([]);
						th = tr[j].childNodes;
						for (k = 0; k < th.length; k++) {
							if (th[k].nodeName.toLowerCase() == 'th') {
								cell = {};
								transferProperty(cell, th[k], 'field');
								transferProperty(cell, th[k], 'name');
								transferProperty(cell, th[k], 'fields');
								transferProperty(cell, th[k], 'cellType');
								transferProperty(cell, th[k], 'options');
								transferProperty(cell, th[k], 'editable');
								transferProperty(cell, th[k], 'draggable');
								transferProperty(cell, th[k], 'formatter');
								transferProperty(cell, th[k], 'width', 'auto');
								cell.name = th[k].innerHTML;
								structure[structure.length - 1].push(cell);
							}
						}
					}
				}
			}
		}
		if (structure.length > 0) {
			params.structure = structure;
		}
	}
}

// if an old widget was found in the DOM we need to remove it, but since we'll be creating new widgets in its
// place we need to preserve at least some of the node structure so new widgets can be created in the correct
// positions.  because of odd behavior of widget.destroy/destroyDescendants/destroyRecursive (even when
// "preserveDom" is true), we'll create a simple set of new nodes and then destroy the old widget.
com.ibm.rpe.DojoHelper.destroyWidgetForReplacement = function(widget) {
	if(widget && widget.domNode) {
		var node = widget.domNode;
		var newDiv = document.createElement(node.nodeName);
		newDiv.setAttribute('id', com.ibm.rpe.Editor.getNodeId(node));

		// recursively add temp div elements for each child widget
		com.ibm.rpe.DojoHelper.createChildMarkerNodes(node, newDiv);

		// add the new temp node structure immediately before the existing widget's node and then destroy the widget
		if (node.parentNode) {
			node.parentNode.insertBefore(newDiv, node);
		}
		widget.destroyRecursive();
	}
}

// recursively creates temporary div elements in the proper DOM structure so the create function can later
// produce a widget hierarchy of the same shape.  any existing widget node with RPE's magic ID will get a 
// corresponding temporary div created for it.
com.ibm.rpe.DojoHelper.createChildMarkerNodes = function(existingNode, newParent) {
	var childNodes = existingNode.childNodes;
	if(childNodes) {
		for(var i=0;i<childNodes.length;i++) {
			var parentPosition = newParent;
			var nodeId = com.ibm.rpe.Editor.getNodeId(childNodes[i]);
			if(nodeId) {
				var newDiv = document.createElement(childNodes[i].nodeName);
				var attributes = childNodes[i].attributes;
				var length = attributes.length;
				for (var j = 0; j < length; j++) {
					newDiv.setAttribute(attributes[j].nodeName, attributes[j].nodeValue);
				}
				newDiv.setAttribute('id', nodeId);
				
				newParent.appendChild(newDiv);
				parentPosition = newDiv;
			}
			else if (childNodes[i].nodeType != 1 && newParent) {
				// Make sure to preserve non-element child nodes (e.g., the text of an anchor) 
				newParent.appendChild(childNodes[i].cloneNode(false));
			}
			com.ibm.rpe.DojoHelper.createChildMarkerNodes(childNodes[i], parentPosition);
		}
	}
}

com.ibm.rpe.DojoHelper.dynamicNodeReplacementPostScript = function(nodeId) {
	var postScriptFunc = function() {
		try {
		// find the newly-created widget
		var widget = dijit.byId(nodeId);
		if(widget) {
			widget.startup();
			if(widget.resize) {
				widget.resize();
			}

			var parentWidget = com.ibm.rpe.DojoHelper.findParentWidget(widget.domNode);
			while(parentWidget) {
				if(parentWidget.resize) {
					parentWidget.resize();
				}
				parentWidget = com.ibm.rpe.DojoHelper.findParentWidget(parentWidget.domNode);
			}

		}
		} catch (e) {
			//log('exception in post: '+e.message);
		}
	};
	
	if(com.ibm.rpe.DojoFrameworkOverrides.usingAsyncLoader) {
		require(["dojo/ready"], 
				function(ready) {
					ready(postScriptFunc);
				
		});	
	} else {
		dojo.addOnLoad(postScriptFunc);
	}

}

com.ibm.rpe.DojoHelper.setDojoWidgetAttribute = function(nodeId, attrName, attrValue) {
	var node = com.ibm.rpe.Editor.findNodeById(nodeId);
	if(typeof(dijit) != "undefined") {
		var widget = dijit.getEnclosingWidget(node);
		if(widget) {
			// create a single param object
			var params = {};
			params[attrName] = attrValue;
			com.ibm.rpe.DojoHelper.fixupWidgetParams(params, widget.declaredClass);
			widget.attr(attrName, params[attrName]);
		}
	}	
}

com.ibm.rpe.DojoHelper.unregisterWidget = function(nodeId) {
	// first check for dojo/dijit
	if(typeof(dijit) != "undefined") {
		var node = com.ibm.rpe.Editor.findNodeById(nodeId);
		if(node) {
			var widget = dijit.byNode(node);
			if(widget) {
				// first handle cases where we need to remove a widget from a parent container
				var parentWidget = dijit.getEnclosingWidget(widget.domNode.parentNode);
				while(parentWidget && !com.ibm.rpe.Editor.getNodeId(parentWidget.domNode)) {
					// keep going up the DOM until we find a widget that has an equivalent in source.
					// this skips over any child widgets that the parent widget created implicitly.
					parentWidget = dijit.getEnclosingWidget(parentWidget.domNode.parentNode);
				}

				if(parentWidget && parentWidget.removeChild) {
					parentWidget.removeChild(widget);
				}

				// finally, destroy the current widget
				widget.destroyRecursive(false);
			}
		}
	}	
}

/**
 * Used to get the bounding box of the targeted node
 * 
 * @param {0} x coordinate
 * @param {1} y coordinate
 * 
 * @return array in the format [x,y,width,height]
 */
com.ibm.rpe.DojoHelper.getBoundingBox = function(x, y) {
	if(typeof(dojo) != "undefined") {
		var box = null;
		var node = document.elementFromPoint(x, y);
		if(node) {
			var rect;
			var widget = dijit.getEnclosingWidget(node);
			if(widget && widget.declaredClass == 'dojox.mobile.TabContainer') {
				var currNode = node;
				while(currNode && currNode != widget.domNode && !dojo.hasClass(currNode, 'mblTabButton') ) {
					currNode = currNode.parentNode;
				}
				rect = currNode.getBoundingClientRect();
			}
			else if (widget && widget.declaredClass == 'dijit.TitlePane') {
				var currNode = node;
				while (currNode && currNode != widget.domNode && !dojo.hasClass(currNode, 'dijitTitlePaneTitle')) {
					currNode = currNode.parentNode;
				}
				rect = currNode.getBoundingClientRect();
			}
			else if(widget && (widget.declaredClass == 'dijit.layout._AccordionButton' || widget.declaredClass == 'dijit.layout._TabButton')) {
				rect = widget.domNode.getBoundingClientRect();
			}
			else {
				rect = node.getBoundingClientRect();
			}
			if(rect) {
				box = [rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top];
			}
		}
		return box;
	}	
}

/**
 * Used to get the child index of the node at the given x,y point in its enclosing parent widget
 * 
 * @param {0} x coordinate
 * @param {1} y coordinate
 * 
 * @return double index child of the node at given x,y coordinate in its enclosing widget,
 * -1 if index can not be calculated, null if error
 */
com.ibm.rpe.DojoHelper.getIndexInParent = function(x, y) {
	if(typeof(dojo) != "undefined") {
		var index = -1;
		var node = document.elementFromPoint(x, y);
		if(node) {
			//com.ibm.rpe.Editor.printNode(node);
			var childWidget = dijit.getEnclosingWidget(node);
			if(childWidget) {
				var parent = childWidget.getParent ? childWidget.getParent() : null;
				if (!parent) {
					var domParent = childWidget.domNode.parentNode;
					if (domParent) {
						parent = dijit.getEnclosingWidget(domParent);
					}
				}

				if(parent && childWidget.declaredClass == 'dijit.layout._AccordionButton' && childWidget.contentWidget) {
					index = parent.getIndexOfChild(childWidget.contentWidget);
				} else if(parent && childWidget.getIndexInParent) {
					index = childWidget.getIndexInParent();
				} else if (parent && parent.getIndexOfChild) {
					index = parent.getIndexOfChild(childWidget);
				} else { // DOM Node]
					var currNode = node;
					//special cases
					if(childWidget.declaredClass == 'dojox.mobile.TabContainer') {
						while(!dojo.hasClass(currNode, 'mblTabButton')) {
							currNode = currNode.parentNode;
						}
					}
					
					//determine index
					var children = currNode.parentNode.childNodes;
					var i = 0;
					for (; i < children.length; i++) {
						if (children[i] == currNode)
							index = i;
					}
				}
			}
		}
		return index;
	}
}

com.ibm.rpe.DojoHelper.getSelectedIndexInContainer = function(nodeId) {
	if(typeof(dojo) != "undefined") {
		var node = com.ibm.rpe.Editor.findNodeById(nodeId);
		if (node) {
			var widget = dijit.getEnclosingWidget(node);
			if (widget && widget.selectedChildWidget && widget.getIndexOfChild) {
				return widget.getIndexOfChild(widget.selectedChildWidget);
			}
		}
	}
	return -1;
}

/**
 * Determine if the node at the given coordinate or any of its parents have
 * a given CSS class applied.
 * 
 * @param {0} x coordinate
 * @param {1} y coordinate
 * @param {2} the name of the CSS class to look for
 * 
 * @return true if the node at the given coordinate or any of its parents
 * have the given CSS class applied, false otherwise, or null if there is
 * an error.
 * 
 */
com.ibm.rpe.DojoHelper.nodeOrParentsHaveCSS = function(x, y, className) {
	if(typeof(dojo) != "undefined") {
		var hasClass = false;
		var node = document.elementFromPoint(x, y);
		if(node) {
			var currNode = node;
			while(!hasClass && currNode && currNode != document.body ) {
				hasClass = dojo.hasClass(currNode, className);
				currNode = currNode.parentNode;
			}
		}
		return hasClass;
	}
}

// This function causes the Dojo parser to re-parse the DOM starting
// at the parent node found for nodeId. The parent node is where Dojo
// will start recursively looking for widgets. The root node itself
// will not be widgetized
com.ibm.rpe.DojoHelper.parseWidget = function(nodeId) {
	if (typeof('dijit') != 'undefined') {
		try {
			var node = com.ibm.rpe.Editor.findNodeById(nodeId);
			if (node && node.parentNode && !dijit.registry.byNode(node)) {
				// Only parse if the node hasn't been parsed yet
				dojo.parser.instantiate([node]);
			}
		}
		catch (e) {
			//log('exception in widget parsing: ' + e.message);
		}
	}
}

// this function modifies the values in the given parameter object to coerce the parameter values
// into the expected data types for the widget contructor.  the given params are all string values initially 
// and some may need to be converted into numbers, booleans, functions, etc.  for declarative widgets that is
// usually done by the dojo parser itself during page load but we also need to provide this functionality
// to support dynamic widget creation based on markup-defined parameter lists.
com.ibm.rpe.DojoHelper.fixupWidgetParams = function(params, dojoType) {
	var widgetClass = dojo.getObject(dojoType);
	if(widgetClass) {
		var objectProto = widgetClass.prototype;
		for(var paramName in params) {
			var val = params[paramName];
			if(val != null) {
				var widgetParamValue = objectProto[paramName];
				if(widgetParamValue != null) {
					// STRING
					if(dojo.isString(widgetParamValue)) {
						// param is already a string so nothing needed
						continue;
					// BOOLEAN
					} else if(typeof(widgetParamValue) == 'boolean') {
						// convert from string to boolean if necessary
						if(val.toLowerCase) {
							params[paramName] = (val.toLowerCase()=='false') ? false : true;
						}
					// DATE
					} else if(widgetParamValue instanceof Date) {
						if (val == '') {
							params[paramName] = new Date('');
						} else if (val == 'now') {
							params[paramName] = new Date();
						} else {
							params[paramName] = dojo.date.stamp.fromISOString(val);
						}
					// NUMBER
					} else if(typeof(widgetParamValue) == 'number') {
						params[paramName] = Number(val);
					// FUNCTION
					} else if(dojo.isFunction(widgetParamValue)) {
						// for editor purposes we don't necessarily care if a dojo widget is fully functional
						// therefore we'll just create a dummy function for now
						params[paramName] = new Function();
					// ARRAY
					} else if(dojo.isArray(widgetParamValue)) {
						params[paramName] = val ? val.split(/\s*,\s*/) : [];
					// URL
					} else if(widgetParamValue instanceof dojo._Url) {
						params[paramName] = dojo.baseUrl + val;
					// OBJECT
					} else {
						// generic object case - handle this last as the default if no other modifications were made
						params[paramName] = dojo.fromJson(val);
					}
				}
			}
		}
	}
	return params;
}


// A browser-neutral implementation to find which array index the given item is in, or -1 if the item is not contained in the array
com.ibm.rpe.DojoHelper.indexOf = function(array, item) {
	var index = -1;
	if(array && item) {
		for(var i=0;i<array.length;i++) {
			if(array[i] == item) {
				return i;
			}
		}
	}
	return index;
}

com.ibm.rpe.DojoHelper.gatherSavedState = function(nodeId) {
	var node = com.ibm.rpe.Editor.findNodeById(nodeId);
	if(node && (typeof(dijit) != 'undefined') && dijit.getEnclosingWidget) {
		var widget = dijit.getEnclosingWidget(node);
		// for containers that have a selected child widget, store that selection
		if (widget && widget.declaredClass == 'dijit.layout.ContentPane') {
			// For a Content Pane, get the parent container to store the state of
			var container = widget.domNode.parentNode;
			if (container) {
				widget = dijit.getEnclosingWidget(container);
				if (widget.declaredClass == 'dijit.layout._AccordionInnerContainer') {
					widget = dijit.getEnclosingWidget(widget.domNode.parentNode);
				}
			}
		}
		if(widget && widget.selectedChildWidget && widget.getIndexOfChild) {
			return com.ibm.rpe.DojoHelper.STATE_SELECTED_INDEX+widget.getIndexOfChild(widget.selectedChildWidget);
		}
		if(widget && (widget.declaredClass == 'dojox.mobile.View' || widget.declaredClass == 'dojox.mobile.ScrollableView' || widget.declaredClass == 'dojox.mobile.SwapView')) {
			return com.ibm.rpe.DojoHelper.STATE_VISIBLE_MOBILE_VIEW+widget.selected;
		}
	}
	return null;
}

com.ibm.rpe.DojoHelper.restoreSavedState = function(nodeId, value) {
	var node = com.ibm.rpe.Editor.findNodeById(nodeId);
	if(node && (typeof(dijit) != 'undefined') && dijit.getEnclosingWidget){
		// need to defer into an addOnLoad function because the widgets might not have
		// be created yet (particularly on IE)
		var restoreStateFunc = function() {
			var widget = dijit.getEnclosingWidget(node);
			if(value && widget) {
				if(value.indexOf(com.ibm.rpe.DojoHelper.STATE_SELECTED_INDEX) > -1) {
					var indexStr = value.substring(com.ibm.rpe.DojoHelper.STATE_SELECTED_INDEX.length);
					var index = parseInt(indexStr);
					if (index >= 0) {
						// if the node is a Content Pane, it's possible that its container parent needs its
						// state restored
						if (widget.declaredClass == 'dijit.layout.ContentPane') {
							var container = widget.domNode.parentNode;
							if (container) {
								widget = dijit.getEnclosingWidget(container);
								if (widget && widget) {
									if (widget.declaredClass == 'dijit.layout._AccordionInnerContainer') {
										widget = dijit.getEnclosingWidget(widget.domNode.parentNode);
									}
								}
							}
						}
						if(widget.selectChild && widget.getChildren) {
							widget.selectChild(widget.getChildren()[index], false);
						}
					}
				} else if(value.indexOf(com.ibm.rpe.DojoHelper.STATE_VISIBLE_MOBILE_VIEW) > -1) {
					var selected = value.substring(com.ibm.rpe.DojoHelper.STATE_SELECTED_INDEX.length);
					if(selected == 'true') {
						widget.selected = true;
						if(dojo.version.minor > 5) {
							widget.show();
						} else {
							dijit.registry.byClass('dojox.mobile.View').forEach(function(widget){ widget.domNode.style.display = "none"; });
							dijit.registry.byClass('dojox.mobile.ScrollableView').forEach(function(widget){ widget.domNode.style.display = "none"; });
							dijit.registry.byClass('dojox.mobile.SwapView').forEach(function(widget){ widget.domNode.style.display = "none"; });
							dijit.registry.byClass('dojox.mobile.FlippableView').forEach(function(widget){ widget.domNode.style.display = "none"; });
							widget.domNode.style.display = "";
						}
					} else {
						widget.selected = false;
					}
				}
			}
		};
		
		if(com.ibm.rpe.DojoFrameworkOverrides.usingAsyncLoader) {
			require(["dojo/ready"], 
					function(ready) {
						ready(restoreStateFunc);
					
			});	
		} else {
			dojo.addOnLoad(restoreStateFunc);
		}
	}
}

com.ibm.rpe.DojoHelper.requireModule = function(module) {
	if(module) {
		if(com.ibm.rpe.DojoFrameworkOverrides.usingAsyncLoader) {
			// for the AMD require() case we're going to pass in a string array.
			// depending on how we're called "module" may already be an array, so check for
			// that and if it's not we'll create a single-item array with the lone module name string
			if(!(module instanceof Array)) {
				module = eval('[ "'+module.replace(/\./g,'/')+'" ]');
			}
			require(module, function(dojo){});
		} else {
			dojo.require(module);
		}
	}
}

com.ibm.rpe.DojoHelper.addCssLink = function(linkPath) {
	var newLink = document.createElement('link');
	newLink.rel = 'stylesheet';
	newLink.type = 'text/css';
	newLink.href = linkPath;
	document.getElementsByTagName('head')[0].appendChild(newLink);
}
