// setup namespace objects
if(!com) {
	var com = {};
}
if(!com.ibm) {
	com.ibm = {};
}
if(!com.ibm.rpe) {
	com.ibm.rpe = {};
}

if(!com.ibm.rpe.Editor) {
	com.ibm.rpe.Editor = function() {}
}

if(!com.ibm.rpe.Editor.config) {
	com.ibm.rpe.Editor.config = {};
}

// define the singleton variables and functions

com.ibm.rpe.Editor.ATTR_NODE_ID = '{0}'; 
com.ibm.rpe.Editor.DOCUMENT_NODE_ID = '{1}';
com.ibm.rpe.Editor.ZOOM_IN_CONTAINER_ID = 'rpe_zoom-container-div';

com.ibm.rpe.Editor.findNodeById = function(idToMatch, rootSearchNode) {
	if(idToMatch == com.ibm.rpe.Editor.DOCUMENT_NODE_ID) {
		return document;
	}
	
	var node;
	if(rootSearchNode) {
		node = rootSearchNode;
	} else {
		node = document;
	}
	var nodesToCheck = new Array();
	
	// do a depth-first walk through the DOM looking for our magic visualizer attribute
	while(node) {
		if(node.childNodes) {
			var i=0;
			for(i=node.childNodes.length-1;i>=0;i--) {
				nodesToCheck.splice(0,0,node.childNodes[i]);
			}	
		}
		if(node.getAttribute) {
			var nodeId = node.getAttribute(this.ATTR_NODE_ID);
			if(nodeId) {
				if(nodeId == idToMatch) {
					return node;
				}
			}
		}
		node = nodesToCheck.splice(0,1)[0];
	}
}

com.ibm.rpe.Editor.getNodeId = function(node) {
	if(node && node.nodeType == 1) {
		return node.getAttribute(this.ATTR_NODE_ID);
	}
	return null;
}

com.ibm.rpe.Editor.setNodeId = function(node, id) {
	if(node && (node.nodeType == 1)) {
		node.setAttribute(this.ATTR_NODE_ID, id);
	}
}

com.ibm.rpe.Editor.getNodeForPoint = function(x, y) {
	var el = document.elementFromPoint(x, y);
	if(el) {
		var id = this.getNodeId(el);
		// fallback search to walk up dom until we find a containing element with an id
		var node = el;
		while(!id && node) {
			id = this.getNodeId(node);
			if(!id) {
				node = node.parentNode;
			}
		}
		if(node.tagName.toUpperCase() == "OPTION") {
			node = node.parentNode;
		}
		if(node.tagName.toUpperCase() == "OPTGROUP") {
			node = node.parentNode;
		}
		return node;
	}
	return null;
}

com.ibm.rpe.Editor.getNodePosition = function(nodeId) {
	var node = this.findNodeById(nodeId);
	var arrPosition = new Array();
	if(node && node.getBoundingClientRect) {
		var rect = com.ibm.rpe.Editor.getBoundingClientRect(node);
		arrPosition[0] = rect.left;
		arrPosition[1] = rect.top;
		arrPosition[2] = rect.right-rect.left;
		arrPosition[3] = rect.bottom-rect.top;
	} else if(node && (node.nodeType == 9)) {
		// this is the document node, so give back a size that's the entire browser area
		arrPosition[0] = -1;
		arrPosition[1] = -1;
		arrPosition[2] = -1;
		arrPosition[3] = -1;
	}
	return arrPosition;
}

// debugging tool to dump the contents of the current node to the console log
com.ibm.rpe.Editor.printNode = function(node, recursive, indentString) {
	if(node) {
		if(!indentString) {
			indentString = '';
		}
		if(node.nodeType == 9) { // document
			var children = node.childNodes;
			if(typeof(recursive) == 'undefined') {
				recursive = true; // if not passed as an arg, assume we want to print the whole subtree
			}
			if(children && recursive) {
				for(var i=0;i<children.length;i++) {
					this.printNode(children[i], recursive, indentString+'  ');
				}
			}
		} else if(node.nodeType == 1) { // element
			var nodeString = indentString+'<'+node.nodeName;
			var attrs = node.attributes;
			for(var i=0;i<attrs.length;i++) {
				var val = node.getAttribute(attrs[i].name);
				if(val) {
					nodeString = nodeString+' '+attrs[i].name+'="'+attrs[i].value+'"';
				}
			}
			log(nodeString+'>');
			
			var children = node.childNodes;
			if(typeof(recursive) == 'undefined') {
				recursive = true; // if not passed as an arg, assume we want to print the whole subtree
			}
			if(children && recursive) {
				for(var i=0;i<children.length;i++) {
					this.printNode(children[i], recursive, indentString+'  ');
				}
			}

			log(indentString+'</'+node.nodeName+'>');
		} else if(node.nodeType == 3) {
			log(indentString+'#text['+node.nodeValue+']');
		}
	}
}


// debugging tool to dump the contents of the current node to the console log
com.ibm.rpe.Editor.serializeNode = function(node, recursive) {
	var output = "";
	if(node) {
		if(node.nodeType == 9) { // document
			var children = node.childNodes;
			if(typeof(recursive) == 'undefined') {
				recursive = true; // if not passed as an arg, assume we want to print the whole subtree
			}
			if(children && recursive) {
				for(var i=0;i<children.length;i++) {
					output = output + this.serializeNode(children[i], recursive);
				}
			}
		} else if(node.nodeType == 1) { // element
			var nodeString = '<'+node.nodeName;
			var attrs = node.attributes;
			for(var i=0;i<attrs.length;i++) {
				var val = node.getAttribute(attrs[i].name);
				if(val) {
					nodeString = nodeString+' '+attrs[i].name+'="'+attrs[i].value+'"';
				}
			}
			nodeString = nodeString + '>';
			
			var children = node.childNodes;
			if(typeof(recursive) == 'undefined') {
				recursive = true; // if not passed as an arg, assume we want to print the whole subtree
			}
			if(children && recursive) {
				for(var i=0;i<children.length;i++) {
					nodeString = nodeString + this.serializeNode(children[i], recursive);
				}
			}

			output = nodeString + '</'+node.nodeName+'>';
		} else if(node.nodeType == 3) {
			output = node.nodeValue;
		}
		return output;
	}
}

// directly runs javascript that would have been in embedded script nodes but had to be hidden inside of divs because of IE innerHTML behaviors
com.ibm.rpe.Editor.executeScripts = function(node) {
	// check for IE and if not, quickly return
	if(!node.swapNode) {
		return;
	}	
	
	var nodesToCheck = new Array();
	while(node) {
		if(node.nodeType==1) {
			var children = node.childNodes;
			if(children) {
				for(var i=children.length-1;i>=0;i--) {
					nodesToCheck.splice(0,0,children[i]);
				}	
			}
			if(node.getAttribute('{2}')) {
				// grab the source text from the pseudo-script node, execute the script, and remove the node
				var text = node.firstChild.nodeValue;
				eval(text);
				if(node.parentNode) {
					node.parentNode.removeChild(node);
				}
			}
		}
		node = nodesToCheck.splice(0,1)[0];
	}
}

com.ibm.rpe.Editor.scrollIntoView = function(node) {
	// get info about the current scroll position of the page (these are measurements of the visible portion in the browser window)
	var docPage = (document.compatMode == "BackCompat") ? document.body : document.documentElement;    // distinguish quirks vs. strict mode doctype issues
	var pageLeft = docPage.scrollLeft;
	var pageTop = docPage.scrollTop;
	var scale = com.ibm.rpe.Editor.config['scale'] ? com.ibm.rpe.Editor.config['scale'] : 1;
	var pageHeight = docPage.clientHeight * scale;
	var pageWidth = docPage.clientWidth * scale;

	// get current node position
	if(node.getBoundingClientRect) {
		var rect = com.ibm.rpe.Editor.getBoundingClientRect(node);

		// if the given node falls partly or entirely outside the currently-visible client area, 
		// we will scroll the document to reveal it.  otherwise do nothing to avoid unnecessary jumping.
		if((rect.top < 0) || (rect.left < 0)) {
			node.scrollIntoView(false);
		} 
		if((rect.bottom > pageHeight) || (rect.right > pageWidth)) {
			node.scrollIntoView(true);
		}
	}
}

com.ibm.rpe.Editor.getComputedStyle = function(node) {
	if(document.defaultView && document.defaultView.getComputedStyle) {
		return document.defaultView.getComputedStyle(node, '');
	} else {
		return node.currentStyle;
	}
}

com.ibm.rpe.Editor.normalizeUri = function(uriString) {
	if(uriString) {
		var segments = uriString.split('/');
		// remove any . segments
		for(var i=0;i<segments.length;i++) {
			if(segments[i] == '.') {
				segments.splice(i, 1);
			}	
		}
		// now remove any .. segments and the previous segment, only if there is a previous segment that is not also ..
		for(var i=0;i<segments.length;i++) {
			if(segments[i] == '..') {
				if((i > 0) && (segments[i-1] != '..')) {
					segments.splice(i-1, 2);
				}
			}
		}
		var returnString = '';
		for(var i=0;i<segments.length;i++) {
			returnString = returnString + segments[i];
			if(i < (segments.length - 1)) {
				returnString = returnString + '/';
			}
		}
		return returnString;
	}
	return '';
}

com.ibm.rpe.Editor.isIE = function() {
	return (document.createEventObject);
}

com.ibm.rpe.Editor.getNodeIdForPoint = function(x, y) {
	var node = com.ibm.rpe.Editor.getNodeForPoint(x, y);
	if(node) {
		//Only allow selection of the body node or any node under it
		var testNode = node;
		while (testNode) {
			if (testNode == document.body) {
				var id = com.ibm.rpe.Editor.getNodeId(node);
				return id;
			}
			testNode = testNode.parentNode;
		}
		//return the body if a selection was made outside the body
		id = com.ibm.rpe.Editor.getNodeId(document.body);
		return id;
	}
}

com.ibm.rpe.Editor.dndDragOver = function(x, y) {
	var node = com.ibm.rpe.Editor.getNodeForPoint(x, y);
	if((node) && (node.getBoundingClientRect)) {
		// notify the editor of the dragover data
		var rect = com.ibm.rpe.Editor.getBoundingClientRect(node);
		var id = com.ibm.rpe.Editor.getNodeId(node);
		return [rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top, id];
	}
	else {
		// If node is not found then user must have dropped on the document, return value to flag result
		return [0, 0, 0, 0, '#document'];
	}
}

com.ibm.rpe.Editor.dndDragOverChildChecker = function(nodeId) {
	var node = com.ibm.rpe.Editor.findNodeById(nodeId);
	if((node) && (node.getBoundingClientRect)) {
		var rect = com.ibm.rpe.Editor.getBoundingClientRect(node);
		return [rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top, nodeId];
	}
}

com.ibm.rpe.Editor.getScrollSizes = function() {
	var docPage = (document && document.compatMode && document.compatMode == "BackCompat") ? document.body: document.documentElement;  // distinguish quirks vs. strict mode doctype issues
	if(docPage) {
		var contentWidth = docPage.clientWidth;
		var totalWidth = docPage.scrollWidth;
		var scrollLeft = document.documentElement.scrollLeft || document.body.scrollLeft; // Webkit always reports 0 for document.documentElement.scrollTop/Left. Webkit bug #9248
			
		var contentHeight = docPage.clientHeight
		var totalHeight = docPage.scrollHeight;
		var scrollTop = document.documentElement.scrollTop || document.body.scrollTop;

		return [contentWidth, totalWidth, scrollLeft, contentHeight, totalHeight, scrollTop];
	}

	return [0,0,0,0,0,0];	
}

com.ibm.rpe.Editor.scrollDocument = function(left, top) {
	if(window.scrollTo) {
		window.scrollTo(left, top);
	} else {
		// scrollTo should always work but leaving our old approach here just in case.  note this doesn't work right for webkit browsers
		var docPage = (document && document.compatMode && document.compatMode == "BackCompat") ? document.body: document.documentElement;  // distinguish quirks vs. strict mode doctype issues
		if(docPage) {
			docPage.scrollLeft=left;
			docPage.scrollTop=top;
		}
	}
}

com.ibm.rpe.Editor.scrollNodeIntoView = function(nodeId) {
	var node = com.ibm.rpe.Editor.findNodeById(nodeId);
	if(node) {
		com.ibm.rpe.Editor.scrollIntoView(node);
	}	
}

com.ibm.rpe.Editor.getClippedPositionRectangle = function(nodeId) {
	var node = com.ibm.rpe.Editor.findNodeById(nodeId);
	if((node) && (node.getBoundingClientRect)) {
		var nodeRect = com.ibm.rpe.Editor.getBoundingClientRect(node);
		// we'll keep track the current bounding rectangle which will be adjusted as clipped occurs against parent nodes
		var clippedRect = { top: nodeRect.top, left: nodeRect.left, bottom: nodeRect.bottom, right: nodeRect.right };

		var testParent = node.parentNode;
		
		if(node == document.body) {
			// if our selected node was the body tag, we're already done, so skip the following loop
			testParent = null;
		}
		
		// loop through all the parent nodes looking for elements to clip the boundaries against, until we hit either the body tag or the 
		// container node that's used for the zoom-in mode
		while ((testParent) && (testParent != document.body) && (testParent.getAttribute) && (com.ibm.rpe.Editor.ZOOM_IN_CONTAINER_ID != testParent.getAttribute('id'))) {
			if (testParent.getBoundingClientRect) {
				var testRect = com.ibm.rpe.Editor.getBoundingClientRect(testParent);
				// any of these overflow styles imply that clipped elements would be all or partly invisible
				var overflow = com.ibm.rpe.Editor.getComputedStyle(testParent).overflow;
				if((overflow == "hidden") || (overflow == "scroll") || (overflow == "auto")) {
					// if any of the current boundary coordinates overhang the parent node we're testing against, clip that side to the new edge
					if(clippedRect.top < testRect.top) {
						clippedRect.top = testRect.top;
					}
					if(clippedRect.bottom > testRect.bottom) {
						clippedRect.bottom = testRect.bottom;
					}
					if(clippedRect.left < testRect.left) {
						clippedRect.left = testRect.left;
					}
					if(clippedRect.right > testRect.right) {
						clippedRect.right = testRect.right;
					}
				}
			}
			testParent = testParent.parentNode;
		}
		return [clippedRect.left, clippedRect.top, clippedRect.right-clippedRect.left, clippedRect.bottom-clippedRect.top];
	} else {
		//If we do not have the node in the document set the values to something safe to avoid displaying an incorrect selection.
		return [0.0, 0.0, 0.0, 0.0];
	}
}

// for zoom-in mode
com.ibm.rpe.Editor.filterVisibleRegion = function(rootNodeId, zoomContainerNodeId) {
	var zoomContainerDiv = document.getElementById(com.ibm.rpe.Editor.ZOOM_IN_CONTAINER_ID);
	var newRootNode = com.ibm.rpe.Editor.findNodeById(rootNodeId);

	// if the zoom container node has already been constructed we're already zoomed in.  
	// just check if the intended zoom root is a direct child of the container, otherwise we have to redo the zoom
	if((zoomContainerDiv == null) || !(zoomContainerDiv && newRootNode && newRootNode.parentNode == zoomContainerDiv)) {
		var childCount = document.body.childNodes.length;
		for(var counter = 0; counter < childCount; counter++) {
			document.body.removeChild(document.body.childNodes[0]);
		}
		zoomContainerDiv = document.createElement("div");
		zoomContainerDiv.setAttribute('id', com.ibm.rpe.Editor.ZOOM_IN_CONTAINER_ID);
		com.ibm.rpe.Editor.setNodeId(zoomContainerDiv, zoomContainerNodeId);
		//For IE we need to rebuild some of the container elements for zoomed in nodes to render correctly 
		if (zoomContainerDiv.swapNode) {
			if(('TD' == newRootNode.nodeName) || ('TH' == newRootNode.nodeName)) {
				trElement = document.createElement("tr");
				trElement.appendChild(newRootNode);
				tbodyElement = document.createElement("tbody");
				tbodyElement.appendChild(trElement);
				tableElement = document.createElement("table");
				tableElement.appendChild(tbodyElement);
				zoomContainerDiv.appendChild(tableElement);
			}
			else if (('TR' == newRootNode.nodeName) || ('THEAD' == newRootNode.nodeName) || ('TFOOT' == newRootNode.nodeName)) {
				tbodyElement = document.createElement("tbody");
				tbodyElement.appendChild(newRootNode);
				tableElement = document.createElement("table");
				tableElement.appendChild(tbodyElement);
				zoomContainerDiv.appendChild(tableElement);
			}
			else if ('TBODY' == newRootNode.nodeName) {
				tableElement = document.createElement("table");
				tableElement.appendChild(newRootNode);
				zoomContainerDiv.appendChild(tableElement);
			}
			else {
				zoomContainerDiv.appendChild(newRootNode);
			}
		}
		else {
			zoomContainerDiv.appendChild(newRootNode);
		}
		document.body.appendChild(zoomContainerDiv);
	}
}

// for zoom-in mode
com.ibm.rpe.Editor.decorateVisibleRegion = function(hasHeightStyle, hasWidthStyle) {
	var zoomContainer = document.getElementById(com.ibm.rpe.Editor.ZOOM_IN_CONTAINER_ID);
	zoomContainer.style.borderWidth = "5px";
	zoomContainer.style.borderStyle = "inset";
	zoomContainer.style.backgroundColor = "white";
	zoomContainer.style.position = "absolute";

	var rootNode = zoomContainer.childNodes[0];
	if (rootNode) {
		// always make the root node visible even if it was a hidden tag when in the whole page
		// we need to use the "!important" priority here to override Dojo's "dijitHidden" class
		// it might be more useful to special-case some extension code for Dojo pages which actually
		// removes this class from the top node
		var inlineStyle = rootNode.style;
		if(inlineStyle.setProperty) {
			inlineStyle.setProperty('top', null, 'important');
			inlineStyle.setProperty('left', null, 'important');
			inlineStyle.setProperty('bottom', null, 'important');
			inlineStyle.setProperty('right', null, 'important');
			inlineStyle.setProperty('display', 'block', 'important');
			inlineStyle.setProperty('position', 'relative', 'important');
			inlineStyle.setProperty('visibility', 'visible', 'important');
		} else if(inlineStyle.setAttribute) {
			// IE special case
			var computed = com.ibm.rpe.Editor.getComputedStyle(rootNode);
			var saveComputed = {};
			//save old style values
			for (x in computed) {
				saveComputed[x] = computed[x];
			}
			//clear old style values, including the problematic important statements
			rootNode.className='';
			//restore saved styles except for display, position and visibility 
			for (x in saveComputed) {
				if (x == 'display') {
					inlineStyle.setAttribute('display', null);
				}
				else if (x == 'top') {
					inlineStyle.setAttribute('top', null);
				}
				else if (x == 'left') {
					inlineStyle.setAttribute('left', null);
				}
				else if (x == 'bottom') {
					inlineStyle.setAttribute('bottom', null);
				}
				else if (x == 'right') {
					inlineStyle.setAttribute('right', null);
				}
				else if (x == 'position') {
					inlineStyle.setAttribute('position', 'relative');
				}
				else if (x == 'visibility') {
					inlineStyle.setAttribute('visibility', 'visibile');
				}
				else {
					inlineStyle.setAttribute(x, saveComputed[x]);
				}
			}
		}
		
		// remove height and width styles so a container will visually expand to the zoomed-in region
		// but only do that if the styles aren't explicitly defined on the source tag.
		// the passed-in params (0 & 1) indicate whether the original source tag had no matching _inline_ style
		// for the appropriate property defined directly on the tag (i.e. a true value indicates it was NOT defined
		// in source and is therefore potentially nullable).
		var computedStyle = com.ibm.rpe.Editor.getComputedStyle(rootNode);
		if((computedStyle.height == null) || (hasHeightStyle && computedStyle.height && inlineStyle.height && (computedStyle.height == inlineStyle.height))) {
			rootNode.style.height = null;
		}
		if((computedStyle.width == null) || (hasWidthStyle && computedStyle.width && inlineStyle.width && (computedStyle.width == inlineStyle.width))) {
			rootNode.style.width = null;
		}
	}

	document.body.style.backgroundColor = "#dddddd";
	
	// need to figure out the size of the "page", which may be either the body element or the document element
	var page = (document.compatMode == "BackCompat") ? document.body : document.documentElement;    // distinguish quirks vs. strict mode doctype issues
	
	// tweak for Dojo mobile pages on IE - for some reason the definition of a viewport in IE switches
	// the "main" page element over to the documentElement even when it should be the body, so we'll do
	// the same for the calculations below
	if(com.ibm.rpe.Editor.isIE()) {
		var metas = document.getElementsByTagName('meta');
		for(var i=0;i<metas.length;i++) {
			if('viewport' == metas[i].getAttribute('name')) {
				page = document.documentElement;
			}
		}
	}

	// tweak the position of the zoom container appropriately for the sizes of the zoomed contents and the visible page area
	if ((page.scrollWidth > window.innerWidth) || (rootNode.scrollWidth + 60 > window.innerWidth)) {
		//contents are wider than the visible area of the browser - let the zoom container size automatically
		zoomContainer.style.width = null;
	} else {
		// manually resize so the zoom area takes up most of the screen
		zoomContainer.style.width = (window.innerWidth - 60) + "px";
	}

	if ((page.scrollHeight > window.innerHeight) || (rootNode.offsetTop + rootNode.scrollHeight + 60 > window.innerHeight)) {
		//contents are taller than the visible area of the browser - let the zoom container size automatically
		zoomContainer.style.height = null;
	} else {
		// manually resize so the zoom area takes up most of the screen
		zoomContainer.style.height = (window.innerHeight - 60) + "px";
	}

	zoomContainer.style.margin = "20px";
}

com.ibm.rpe.Editor.isRTL = function(nodeId) {
	var node = com.ibm.rpe.Editor.findNodeById(nodeId);
	if(node) {
		while(node && node.getAttribute) {
			var dir = node.getAttribute('dir');
			if(dir && ('rtl' == dir)) {
				return true;
			} else if(dir && ('ltr' == dir)) {
				return false;
			}
			node = node.parentNode;
		}
	}
	return false;
}

com.ibm.rpe.Editor.repaintOnSizeChange = function(nodeId) {
	var node = com.ibm.rpe.Editor.findNodeById(nodeId);
	if(node) {
		var computedStyle = com.ibm.rpe.Editor.getComputedStyle(node);
		var origHeight = computedStyle.height;
		var origWidth = computedStyle.width;

		var checkInterval = 100;  // milliseconds between checks
		var checkCount = 10;  // max number of checks for size change  

		var timerFunc = function(h, w, count) {
			if(count == 0) {
				return;
			}
			var computedStyle = com.ibm.rpe.Editor.getComputedStyle(node);
			if((h != computedStyle.height) || (w != computedStyle.width)) {
				IBMRPE_request_repaint();
			} else {
				setTimeout(function() { timerFunc(h, w, count-1) }, checkInterval);
			}
		}
		
		setTimeout(function() { timerFunc(origHeight, origWidth, checkCount) }, checkInterval);
	}
}

com.ibm.rpe.Editor.repaint = function(millisDelay) {
	var repaintFunc = function() {
		IBMRPE_request_repaint();
	}
	if(millisDelay) {
		setTimeout(repaintFunc, millisDelay);
	} else {
		repaintFunc();
	}
}

/**
 * Scale the contents of the web page by the factor specified in the argument
 */
com.ibm.rpe.Editor.scaleDeviceContents = function(factor) {
	if (factor) {
		// Scale and reposition the body tag
		var prefix, style;
		if (document.documentElement && (style = document.documentElement.style)) {
			if (typeof style.transform != 'undefined') {
				prefix = 'transform';
			}
			else if (typeof style.webkitTransform != 'undefined') {
				prefix = 'webkitTransform';
			} 
			else if (typeof style.MozTransform != 'undefined') {
				prefix = 'MozTransform';
			}
			else if (typeof style.msTransform != 'undefined') {
				prefix = 'msTransform';
			}
		}
		if (document.body) {
			if (prefix) {
				document.body.style[prefix] = 'scale('+factor+')';
				document.body.style[prefix+'Origin'] = '0px 0px';
			}
			else {
				//document.body.style['filter'] = 'progid:DXImageTransform.Microsoft.Matrix(M11='+factor+', M12=0, M21=0, M22='+factor+', SizingMethod="auto expand")';
				document.body.style['zoom'] = factor;
			}
			com.ibm.rpe.Editor.config['scale'] = factor;
		}
	}
}

/**
 * Called by the sync helper after modifications to the body's style attribute.  We need to rescale the contents
 * of the browser if we're in scaling mode.
 */
com.ibm.rpe.Editor.restoreDeviceScaling = function() {
	var scale = com.ibm.rpe.Editor.config['scale'];
	if(scale) {
		com.ibm.rpe.Editor.scaleDeviceContents(scale);
	}
}

com.ibm.rpe.Editor.supportsCSSTransform = function() {
	if (document.documentElement && (style = document.documentElement.style)) {
		return typeof style.transform != 'undefined' || typeof style.webkitTransform != 'undefined' || typeof style.MozTransform != 'undefined' || typeof style.msTransform != 'undefined';
	}
	return false;	
}

com.ibm.rpe.Editor.isMozilla = function() {
	return (document.documentElement && document.documentElement.style && (typeof document.documentElement.style.MozTransform != 'undefined'));
}

/**
 * Returns the bounding client rectangle for the specified node. Mozilla bounding rectangles
 * need to be scaled by the editor's scaling factor
 */
com.ibm.rpe.Editor.getBoundingClientRect = function(node) {
	var rect = {};
	if(node && node.getBoundingClientRect) {
		rect = node.getBoundingClientRect();
		if(com.ibm.rpe.Editor.isMozilla()) {
			var scale = com.ibm.rpe.Editor.config['scale'] ? com.ibm.rpe.Editor.config['scale'] : null;
			if(scale) {
				// when in device scaling mode on Firefox, we need to scale the pixel coordinates we get from the
				// browser while also factoring out the non-zero position of the body element
				var bodyLeftEdge = document.body.getBoundingClientRect().left;
				var bodyTopEdge = document.body.getBoundingClientRect().top;
				rect = {
						left: Math.round((rect.left - bodyLeftEdge) * scale + bodyLeftEdge),
						top: Math.round((rect.top - bodyTopEdge) * scale + bodyTopEdge),
						right: Math.round((rect.right - bodyLeftEdge) * scale + bodyLeftEdge),
						bottom: Math.round((rect.bottom - bodyTopEdge) * scale + bodyTopEdge)
				};
			}
		}
		
		// Handling a special case for the <option> and <optgroup> nodes, because at least in webkit
		// its getBoundingClientRect is [0,0,0,0]. So, when an option or optgroup node is selected,
		// its parent (<select> node) is showing as selected in Design Pane.
		if (node.tagName.toUpperCase() == "OPTION" || node.tagName.toUpperCase() == "OPTGROUP") {
			var select = node.parentNode;
			if(select.tagName.toUpperCase() == "OPTGROUP") {
				select = select.parentNode;
			}
		    rect = com.ibm.rpe.Editor.getBoundingClientRect(select);
		}
	}
	return rect;
}

// Calculate a style property on a given node
com.ibm.rpe.Editor.getStyle = function(node, styleProp) {	
	var camelize = function(str) {
		return str.replace(/\-(\w)/g, function(str, letter) {
			return letter.toUpperCase();
		});
	};

	if (node.currentStyle) {
		return node.currentStyle[camelize(styleProp)];
	} else if (document.defaultView && document.defaultView.getComputedStyle) {
		return document.defaultView.getComputedStyle(node, null)
				.getPropertyValue(styleProp);
	} else {
		return node.style[camelize(styleProp)];
	}
}

// Get bounding client rect on an <options> node
com.ibm.rpe.Editor.getOptionBoundingClientRect = function(clippedRect, option) {
	var select = option.parentNode;
	if(select.tagName.toUpperCase() == "OPTGROUP") {
		select = select.parentNode;
	}
    var rect = com.ibm.rpe.Editor.getBoundingClientRect(select);
    
	if(clippedRect.left == 0 && clippedRect.top == 0 && clippedRect.right == 0 && clippedRect.bottom == 0){
	    var i = option.index;
	    var fontSize = com.ibm.rpe.Editor.getStyle(option, 'font-size');
	    var padding = com.ibm.rpe.Editor.getStyle(option, 'padding-top');    
	    var border = com.ibm.rpe.Editor.getStyle(option, 'border-top-width');
	    var margin = com.ibm.rpe.Editor.getStyle(option, 'margin-top');
	    var height = parseInt(fontSize.replace ('pt', '')) +
	    	parseInt(padding.replace ('px', '')) + 
	    	parseInt(border.replace ('medium', '0')) +
	    	parseInt(margin.replace ('auto', '0')) + 4; // Cannot get the height of the option element, so calculate it
	    clippedRect = { 
	    	top: rect.top + ((i * height) - select.scrollTop), 
	    	left: rect.left, 
	    	bottom: rect.top + (((i + 1) * height) - select.scrollTop), 
	    	right: rect.right };
	}
	
    if(clippedRect.top >= rect.top) {
    	if(clippedRect.bottom <= rect.bottom) {
    		return clippedRect;
    	} else {
    		clippedRect.bottom = rect.bottom;
    	}
    } else {
    	clippedRect.top = rect.top;
    }
    return clippedRect;
}

// Get bounding client rect on an <optgroup> node
com.ibm.rpe.Editor.getOptGroupBoundingClientRect = function(rect, opt) {
    var select = opt.parentNode;
    var rectSelect = com.ibm.rpe.Editor.getBoundingClientRect(select);    
    if(rect.top >= rectSelect.top) {
    	if(rect.bottom <= rectSelect.bottom) {
    		return rect;
    	} else {
    		rect.bottom = rectSelect.bottom;
    	}
    } else {
    	rect.top = rectSelect.top;
    }
    return rect;
}