// define standard DOM-sync functions - this is not in the main Editor object because it's not official API 
// setup namespace objects
if(!com) {
	var com = {};
}
if(!com.ibm) {
	com.ibm = {};
}
if(!com.ibm.rpe) {
	com.ibm.rpe = {};
}

if(!com.ibm.rpe.SyncHelper) {
	com.ibm.rpe.SyncHelper = function() { }
}

com.ibm.rpe.SyncHelper.TEXT_MARKER = '[t]';
com.ibm.rpe.SyncHelper.ELEMENT_MARKER = '[e]';

com.ibm.rpe.SyncHelper.addNode = function(nodeId, nextSiblingId, parentNodeName, nodeSource, extraParentNodesCount) {
	var node = com.ibm.rpe.Editor.findNodeById(nodeId);

	if(node) {
		// find the right child node to insert before, or null if we're appending
		var nextSib = com.ibm.rpe.Editor.findNodeById(nextSiblingId, node);
		
		// create the contents inside a fake element
		var tempParent = document.createElement(parentNodeName);
		tempParent.innerHTML = nodeSource;
		
		// move down the DOM to the correct child for cases where we had to wrap extra nodes around the desired new content
		for(i=0;i<extraParentNodesCount;i++) {
			var children = tempParent.childNodes;
			// move down into the first child element (there can be no text nodes in the passed-in source)
			tempParent = children[0];
		}			
		
		// extract the new node contents
		var newNode = tempParent.childNodes[0];

		// special cases for table node children.  if the content is a tr, we'll need to reposition "node" to be the tbody if it's pointing at the table tag itself
		if('TR' == newNode.nodeName) {
			if('TABLE' == node.nodeName) {
				node = node.getElementsByTagName('TBODY')[0];
			}
		}

		// insert it under the parent in the right position
		// note: though insertBefore should handle nextSib being null, there's a type mismatch error so do the explicit check here
		if(nextSib) {
			nextSib.parentNode.insertBefore(newNode, nextSib);
		} else {
			//If document node then sub in document.body
			if (node.nodeType == 9) {
				document.body.appendChild(newNode);
			}
			else {
				node.appendChild(newNode);
			}
		}
		
		// for IE, we can't have embedded <script> nodes in the inserted nodes so go execute the script code now (they've been previously doctored when transforming the nodes)
		com.ibm.rpe.Editor.executeScripts(newNode);
	}
}

// returns true if a node with the given ID was found and removed from the DOM, false if the node was absent
com.ibm.rpe.SyncHelper.removeNode = function (nodeId) {
	if(!com.ibm.rpe.Editor.isIE()) {
		// "regular" non-IE case
		var node = com.ibm.rpe.Editor.findNodeById(nodeId);
		if(node) {
			node.parentNode.removeChild(node);
			return true;
		}
	} else {
		// IE case (not surprisingly IE causes us all sorts of problems...)
		// for cases where tags are partially complete in source (ex. while the user is in the middle of typing them) or otherwise "unknown" tags are used, IE's DOM has 
		// confused or missing information.  therefore when we come back to remove nodes for subsequent DOM edits, certain things get "lost".
		//
		// say you have a <p> tag and start typing a button tag before it.  In SSE's DOM the button tag (even while partial) wraps the paragraph.  So the source
		// sent to the browser when you've typed the first few letters is:
		//			<but>
		//				<p>some text here</p>
		//			</but>
		//
		// there are two scenarios:
		//	1) page load time:  the source above when parsed at load time becomes a DOM structure like this:
		//			<but></but>
		//			<p>some text here</p>
		//			</but><//but>			// <---- yes, seriously
		//
		//		thus, a wrapper tag has become broken into two peers in the DOM, only one of which (the open tag) has an RPE editor ID attribute.  
		//		to handle this we'll need to look for a	sibling element with a tag name that looks like an end tag and delete everything between these two boundaries, inclusive.
		//
		//  2) live update while typing:   when injected into the browser DOM dynamically using innerHTML, the unknown tag is stripped away entirely, so the result is simply:
		//			<p>some text here</p>
		//
		//		this is even more challenging because the wrapper node is simply not there.  
		// 		it will be solved by other means of correcting the DOM

		
		// handle the #1 scenario by looking for a bogus "end tag" as a sibling in the DOM
		var node = com.ibm.rpe.Editor.findNodeById(nodeId);
		if(node) {
			var endTagName = '/' + node.nodeName;
			var endNode = node;
			var tempNode = node.nextSibling;
			while(tempNode != null) {
				if(endTagName == tempNode.nodeName) {
					endNode = tempNode;
					break;
				}
				tempNode = tempNode.nextSibling;
			}
			if(node == endNode) {
				// the node is intact so can be deleted normally
				node.parentNode.removeChild(node);
				return true;
			} else {
				// we need to remove a range of nodes
				var nodeToRemove = node;
				var nextToRemove = node;
				while(nextToRemove != null) {
					nextToRemove = nodeToRemove.nextSibling;
					nodeToRemove.parentNode.removeChild(nodeToRemove);
					nodeToRemove = nextToRemove;
					
					if(endNode == nextToRemove) {
						// end of the range
						nextToRemove.parentNode.removeChild(nextToRemove);
						return true;
					}
				}
			}
		}
	}
	// we didn't already find and remove a node, so return false - this indicates to the calling function that the node wasn't in
	// the browser DOM and we'll need to keep looking for children until the whole subtree is removed
	return false;
}

com.ibm.rpe.SyncHelper.modifyTextNodes = function (nodeId, expectedChildNodes) {
	var parent = com.ibm.rpe.Editor.findNodeById(nodeId);
	if(parent) {
		if(parent.nodeType == 9) {
			// if the parent node is the document itself, use the body element instead (page fragment case)
			parent = document.body;
		}
		
		var mode = (com.ibm.rpe.Editor.isIE())? document.documentMode : 0;		
				
		// modify text nodes to trim whitespace, as required by the position in the DOM
		for(i=0;i<expectedChildNodes.length;i++) {
			var prefix = expectedChildNodes[i].substring(0, 3);
			if(prefix == com.ibm.rpe.SyncHelper.TEXT_MARKER) {
				expectedText = expectedChildNodes[i].substring(3);
				// fix for the preview pane formatting issue
				if(mode == "5.0"){
						expectedText = expectedText.replace(/(\s+)/g," ");		
				}
				var newText = com.ibm.rpe.SyncHelper.trimText(parent, expectedText);
				expectedChildNodes[i] = prefix + newText;
			}
		}

		// we keep track of either the expected textual value or the id of an expected element
		var expectedText = null;
		var expectedNodeId = null;
		
		// walk through all the existing child nodes and compare them to the nodes we're expecting
		var childIndex = 0;
		var expectedChildIndex = 0;
		while(childIndex < parent.childNodes.length) {
			var child = parent.childNodes[childIndex++];

			if(expectedChildIndex >= expectedChildNodes.length) {
				// we're looking at something in the browser DOM that's past the end of our expected content - remove any text nodes here
				if(child.nodeType == 3) {
					parent.removeChild(child);
					childIndex--;
					continue;
				}
			}

			// see if we're expecting an element or a text node
			var expectedValue = expectedChildNodes[expectedChildIndex];
			if(expectedValue) {
				var prefix = expectedValue.substring(0, 3);
				if(prefix == com.ibm.rpe.SyncHelper.TEXT_MARKER) {
					expectedText = expectedValue.substring(3);
					expectedNodeId = null;
				} else if(prefix == com.ibm.rpe.SyncHelper.ELEMENT_MARKER) {
					expectedText = null;
					expectedNodeId = expectedValue.substring(3);
				}
				expectedChildIndex++;
				
				if(child.nodeType == 1) {
					// we have an element, let's see what we're expecting
					if(expectedNodeId) {
						var expectedNode = com.ibm.rpe.Editor.findNodeById(expectedNodeId);
						if(expectedNode) {
							// we have found the node we're looking for, now we'll check if it's in the position we're currently checking
							if(expectedNode.parentNode && (expectedNode.parentNode == parent) && (expectedNodeId != com.ibm.rpe.Editor.getNodeId(child))) {
								// we've found an element in the browser, but it's not what we're expecting.  this can potentially happen if something injected extra DOM 
								// elements that don't map to the source DOM.  to handle, we'll skip this node and back up the index for checking the expected children
								expectedChildIndex--;
							}
						} else {
							// we didn't find the expected node in the DOM at all - this can happen if a visualization suppresses the output of a source tag in the browser DOM.
							// for this case, we'll back up the index of the child in the browser side so we continue to check against that element next time
							childIndex--;
						}
						// we're expecting an element, so continue
						continue;
					} else {
						// we need a text node at this position, so add it
						parent.insertBefore(document.createTextNode(expectedText), child);
					}
				} else if(child.nodeType == 3) {
					if(expectedNodeId) {
						// we expected an element, so we need to delete this text node
						parent.removeChild(child);
						// now back up the counts so we continue checking from the same relative positions
						childIndex--;
						expectedChildIndex--;
					} else {
						// make sure the text node's value is what we wanted
						child.nodeValue = expectedText;
					}
				}
			}
		}

		if(expectedChildIndex <= expectedChildNodes.length) {
			// we're expecting more child nodes than we've handled so far - we may need to add an extra text node at the end
			var prefix = expectedChildNodes[expectedChildIndex].substring(0, 3);
			if(prefix == com.ibm.rpe.SyncHelper.TEXT_MARKER) {
				expectedText = expectedChildNodes[expectedChildIndex].substring(3);
			} else {
				expectedText = null;
			}
			if(expectedText) {
				// special case for IE - script and style tags are handled differently and don't use child text nodes to define their contents programmatically
				// script tags are handled asynchronously by calling for a browser reload so we can ignore them here, but we need to handle <style> correctly
				if(com.ibm.rpe.Editor.isIE() && ('STYLE' == parent.nodeName)) {
					parent.styleSheet.cssText = expectedText;
				} else {
					// normal case for all browsers and all other tags
					parent.appendChild(document.createTextNode(expectedText));
				}

			}
		}

	}
}

com.ibm.rpe.SyncHelper.trimText = function (parent, textValue) {
	var omitWhitespaceNodes = true;
//	var omitWhitespaceNodes = false;
//	if(parent == document.body) {
//		// for direct children of the body element, we need to omit any text nodes that trim to whitespace
//		// this matches the browser's behavior on the initial page load
//		omitWhitespaceNodes = true;
//	}
//	
//	// also trim text around and in table rows/cells
//	if(('TR' == parent.nodeName) || 
//		('TD' == parent.nodeName) || 
//		('TH' == parent.nodeName) || 
//		('THEAD' == parent.nodeName) || 
//		('TFOOT' == parent.nodeName) || 
//		('TBODY' == parent.nodeName) || 
//		('TABLE' == parent.nodeName))	{
//		omitWhitespaceNodes = true;
//	}
//	
//	if(('UL' == parent.nodeName)) {
//		omitWhitespaceNodes = true;
//	}
	
	//FIXME - get the full list of HTML tags where we want to disregard whitespace text and check for them above
	// UL & LI ??
	
	if(omitWhitespaceNodes) {
		// if the text value trims to nothing (i.e. is pure whitespace), replace it with a single space.
		// IE generally ignores text nodes but other browsers (at least firefox) may show a visual difference 
		// just by the presence of a text node at all.  so we'll make sure there's a text node there, but 
		// collapse it down to a single character
		if(textValue.replace(/^\s+|\s+$/g,'') == '') {
			//Do not pad with a space on IE because browser will ignore white space on refresh.
			if(com.ibm.rpe.Editor.isIE()) {
				return '';
			}
			else {
				return ' ';
			}
		}
	}
	
	return textValue;
}

com.ibm.rpe.SyncHelper.removeNodeAttribute = function (nodeId, attrName) {
	var node = com.ibm.rpe.Editor.findNodeById(nodeId);
	if(node) {
		// have to clear out the value first, then remove the attribute
		node.setAttribute(attrName, '');
		node.removeAttribute(attrName);
		if((document.body == node) && (attrName.toLowerCase() == "style")) {
			com.ibm.rpe.Editor.restoreDeviceScaling();
		}
	}
}

com.ibm.rpe.SyncHelper.setNodeAttribute = function (nodeId, attrName, attrValue) {
	var node = com.ibm.rpe.Editor.findNodeById(nodeId);
	if(node) {
		//For IE need to set attributes for class and style differently.
		if (node.swapNode) {
			//Check for style attribute
			if (attrName.toLowerCase() == "style") {
				node.style.setAttribute("cssText", attrValue);
			}
			//Check for class attribute
			else if (attrName.toLowerCase() == "class") {
				node.setAttribute("className", attrValue);
			}
			//For all other attributes being set
			else {
				node.setAttribute(attrName, attrValue);
			}
		}
		else {
			//For Firefox one call sets the style or the class.
			node.setAttribute(attrName, attrValue);
		}
		if((document.body == node) && (attrName.toLowerCase() == "style")) {
			com.ibm.rpe.Editor.restoreDeviceScaling();
		}
	}
}

com.ibm.rpe.SyncHelper.removeAllChildNodes = function (nodeId) {
	var node = com.ibm.rpe.Editor.findNodeById(nodeId);
	if(node) {
		while(node.hasChildNodes()) {
			node.removeChild(node.firstChild);
		}
	}
}

com.ibm.rpe.SyncHelper.updateCssLinks = function(linkSrc, inMemoryCssChange, selectedNodeIds, dynamicSuffix, resourceChangeString) {
	var links = document.getElementsByTagName('link')
	for(var counter = 0; counter < links.length; counter++) {
		var refUri = links[counter].getAttribute('href');
		if(refUri) {
			// normalize the href of the style that we're checking so weird (but valid) forms like ./myfile.css don't confuse the check.
			// just checking the beginning of the string because previously-reloaded links will already have the RPE suffix
			if(com.ibm.rpe.Editor.normalizeUri(refUri).indexOf(linkSrc) == 0) {  
				// found the link we're going to change
				var link = links[counter];
				// replacing the link with a new one to force a reload and reparse
				var addLink = document.createElement('link');
				if (inMemoryCssChange) {
					addLink.setAttribute('href', linkSrc + '_' + new Date().valueOf() + dynamicSuffix);  // form becomes: foo.css_1234_dyncss
				} else {
					addLink.setAttribute('href', linkSrc + resourceChangeString + new Date().valueOf());  // form becomes: foo.css_rpe_1234
				}
				addLink.setAttribute('rel','stylesheet');
				addLink.setAttribute('type', 'text/css');
				// need to replace the link into the same dom location so CSS cascade order is preserved
				link.parentNode.replaceChild(addLink, link);  
			}
		}
	}

	// the changes to css may cause a change in sizing for some elements, so set up the timers to check for size
	// changes for all the currently-selected nodes.
	if(selectedNodeIds) {
		for(var i=0;i<selectedNodeIds.length;i++) {
			com.ibm.rpe.Editor.repaintOnSizeChange(selectedNodeIds[i]);
		}
	}
}