// if a table or table cell in the browser DOM doesn't otherwise have an explicit border styling, add one so we can see it (part of non-wysiwyg visualization replacement)
var node = com.ibm.rpe.Editor.findNodeById('{0}');
if(node) {
	
	// we want to set the border style, width, and color if they're not already set.
	// we'll look at the computed style of the element and see if these styles are using their
	// default values.  only then will we apply changes.
	// also, if the table element's "border" attribute is set we won't style that node 
	
	// there are lots of browser differences and quirks to consider here:
	// 	-both Firefox and IE take a general style like "border-style" and shove the value
	// 		into each of the border-left-style, border-top-style, etc. and then unset the "border-style" 
	//		setting (as far as I can tell the computed style never has a general "border-style" set)
	//  -the default values for a plain unstyled table differ by browser:
	// 		border-*-style = 'none' for both
	//		border-*-width = '0px' for FF, 'medium' for IE
	//		border-*-color = 'rgb(128, 128, 128)' for FF, '#d4d0c8' for IE

	
	// to make this easier, we're just going to look for a 'none' border style and set all three of our values
	var applyStyle = function(node) {
		if(!node.border || (node.border == '')) {
			var style = com.ibm.rpe.Editor.getComputedStyle(node);
			var leftStyle = style.borderLeftStyle;
			var topStyle = style.borderTopStyle;
			var rightStyle = style.borderRightStyle;
			var bottomStyle = style.borderBottomStyle;
			if( (!leftStyle && !topStyle && !rightStyle && !bottomStyle) ||
				((leftStyle == 'none') && (topStyle == 'none') && (rightStyle == 'none') && (bottomStyle == 'none')) )	{
				if('TABLE' == node.nodeName) {
					node.style.borderStyle='dotted';
				} else {
					node.style.borderStyle='dashed';
				}
				node.style.borderWidth='1px';
				node.style.borderColor='black';
			}
		}
	}
	
	// save the original node - we'll style this last so cells underneath it don't inherit our changes yet
	var originalNode = node;
	
	// now walk the dom underneath this node
	// in cases where a tag visualizer returns a table, only the top-most node will have a vizid, so we'll look at descendants until we find a node that has one of these ids
	// or other table/cell tags that need decorations
	var nodesToCheck = new Array();

	if(node.childNodes) {
		var i=0;
		for(i=node.childNodes.length-1;i>=0;i--) {
			nodesToCheck.splice(0,0,node.childNodes[i]);
		}	
	}
	node = nodesToCheck.splice(0,1)[0];
	while(node) {
		var vizid = com.ibm.rpe.Editor.getNodeId(node);
		if((vizid == null) || (vizid == '')) {
			if(('TABLE' == node.nodeName) || ('TD' == node.nodeName) || ('TH' == node.nodeName)) {
				applyStyle(node);
			}
			
			if(node.childNodes) {
				var i=0;
				for(i=node.childNodes.length-1;i>=0;i--) {
					nodesToCheck.splice(0,0,node.childNodes[i]);
				}	
			}
		}
		
		node = nodesToCheck.splice(0,1)[0];
	}

	applyStyle(originalNode);
	
}